/*
 * Christophe Lucas <clucas@rotomalug.org>
 *
 * Setup Intel 8254 PIC
 */

#include <testos/types.h>
#include <testos/ioports.h>
#include <video/bochs.h>
#include <video/x86_video.h>
#include <x86/irq.h>
#include <x86/i82C54.h>

/* 
 * PIT I82C54: 
 * puce cadence  1193182 MHz
 * 1193182/65536 = 18.2 Hz
 * Je veux un tick  100Hz:
 * 1193182/100 = 11931.82 ~= 11932
 * => 0x2E9C
 */

#define I82C54_MAX_FREQ		1193182

/* 
 * On utilise uniquement le compteur 0 en utilisant le
 * le mode 2(Clock Rate) pour gnrer un timer
 * 0x0<<0 : binary code
 * 0x2<<1 : mode 2
 * 0x3<<4 : on lit/crit d'abord l'octet de poids faible 
 * 	    puis celui de poid fort
 * 0x0<<6 : compteur 0
 *
 */
#define PROGRAM_COUNTER0	( 0x0<<6 | 0x3<<4  | 0x2<<1 | 0x0<<0)

static ui32_t jiffies = 0 ;

static void timer_handler( int irq )
{
	jiffies++;
	bochs_printf("jiffies value is : %d\n", (int)jiffies );
}

ret_t setup_pit(ui16_t freq)
{
	int nb_ticks = 0;
	
	if (freq == 0)
		return -EINVAL;
		
	nb_ticks = I82C54_MAX_FREQ / freq;
	
	if (nb_ticks == 65536)
		nb_ticks = 0;

	/* on programme le fonctionnement du 82C54 */
	outb(PROGRAM_COUNTER0, I82C54_MODE_PORT);
	/* On crit le LSB du mot initial */
	outb(0x00ff&nb_ticks,I82C54_COUNTER0_PORT);	
	/* on crit le MSB du mot initial */
	outb(nb_ticks>>8,I82C54_COUNTER0_PORT);

	setup_irq_routine(IRQ_TIMER, timer_handler);

	return OK;
}
