/*
 * segment.h:
 *
 * 	Christophe Lucas <clucas@rotomalug.org>
 */

#ifndef __SEGMENT_H__
#define __SEGMENT_H__

#include <testos/types.h>
#include <testos/errno.h>

#define SEG_NULL	0
#define SEG_KCODE	1
#define SEG_KDATA	2

struct gdtr_register_t {
	ui16_t limit;
	ui32_t base_address;
} __attribute__((packed, aligned(8)));

struct segment_descriptor_t {
	ui16_t limit_0_15;
	ui16_t base_address_0_15;
	ui8_t base_address_16_23;
	ui8_t segment_type:4;
	ui8_t descriptor_type:1;
	ui8_t dpl:2;
	ui8_t present:1;
	ui8_t limit_16_19:4;
	ui8_t custom:1;
	ui8_t mode_64_bits_code:1;
	ui8_t segment_32_bits:1;
	ui8_t granularity:1;
	ui8_t base_address_24_31;
} __attribute__((packed, aligned(8)));

struct segment_selector_t {
	ui8_t dpl:2;
	ui8_t table_indicator:1;
	ui16_t index:13;
} __attribute__((packed, aligned(8)));

#define BUILD_GDT_ENTRY(privilege, is_code)		\
	((struct segment_descriptor_t) {		\
		.limit_0_15 = 0xffff,			\
		.base_address_0_15 = 0,			\
		.base_address_16_23 = 0,		\
	 	/* 					\
		 * code = Execute/Read/accessed		\
		 * data = R/W/Accessed			\
		 */					\
		.segment_type = (is_code) ? 0xb : 0x3,	\
		/*					\
		 * System descriptor = 0		\	
		 * Code/Data desc = 1			\
		 */					\
	 	.descriptor_type = 1,			\
		/* Ring 0 GDTE */			\
		.dpl = ((privilege) & 0x3),		\
		.present = 0x1,				\
		.limit_16_19 = 0xf,			\
		.custom = 0,				\
		.mode_64_bits_code = 0,			\
		.segment_32_bits = 1,			\
		/* 1 == 4K pages */			\
		.granularity = 1,			\
		.base_address_24_31 = 0			\
	 })

#define BUILD_SEGMENT_SELECTOR(dpl, ti, index)		\
	  	(  (((dpl) & 0x3)  << 0) 		\
		|  ((ti) << 2)				\
		|  ((index) << 3) )

extern ret_t setup_segmentation( void );
#endif
